
cdef extern from "timecode.h":
    cdef enum TC_TYPE:
        TC_TYPE_ERR, TC_TYPE_30, TC_TYPE_25,
        TC_TYPE_24, TC_TYPE_60, TC_TYPE_30DF

    ctypedef struct tc_s:
            unsigned char   hour
            unsigned char   min
            unsigned char   sec
            unsigned char   frame
            unsigned char   type
            unsigned char   user[4]

    unsigned int tc_frame_to_timecode(unsigned int frame_number,tc_s *timecode)
    unsigned int tc_timecode_to_frame (tc_s *timecode)
    unsigned int tc_timecode_to_string(tc_s *timecode, char *timecode_string)
    unsigned int tc_string_to_timecode(char *timecode_string, tc_s *timecode)
    unsigned int tc_type_to_string(unsigned int tc_type, char *type)
    unsigned int tc_string_to_type(char *type)
    unsigned int tc_string_to_frame(char *sframe)
    unsigned int tc_frame_to_string(unsigned int frame, char *sframe)


cdef class Timecode:
    """

    Math methods are not yet provided on Timecode objects (patches
    welcome). Use int(tc) to get the number of frames represented by a
    Timecode.
    
    """
    cdef tc_s tc
    def __new__(self,*args,**kwargs):
        """
        The args to the constructor can be any of:
          a number of frames
          a timecode string like 00:00:00:00
          four arguments giving the hour,min,sec,frames of the time

        30DF is the default type. For a different type, add a keyword
        argument type="...". Allowed timecode types are
        '30','25','24','60', and '30DF'.
     
        """
        cdef char *arg0

        # compatibility with old argument name
        if 'timecodetype' in kwargs: 
            kwargs['type'] = kwargs['timecodetype']

        kwargs.setdefault("type","30DF")

        t = kwargs['type']
        if t == '30': self.tc.type = TC_TYPE_30
        if t == '25': self.tc.type = TC_TYPE_25
        if t == '24': self.tc.type = TC_TYPE_24
        if t == '60': self.tc.type = TC_TYPE_60
        if t == '30DF': self.tc.type = TC_TYPE_30DF
     
        if len(args)==4:
            self.tc.hour,self.tc.min,self.tc.sec,self.tc.frame = args
        elif len(args)==1 and isinstance(args[0],(str,unicode)):
            # spelled-out timecode
            arg0 = args[0]
            tc_string_to_timecode(arg0, &self.tc)
        elif len(args)==1 and isinstance(args[0],(int,long)):
            tc_frame_to_timecode(args[0],&self.tc)
            
    def __str__(self):
        #return tc_timecode_to_string(self.tc)
        return "%02u:%02u:%02u:%02u" % (self.tc.hour,self.tc.min,
                                        self.tc.sec,self.tc.frame)
    def __int__(self):
        return tc_timecode_to_frame(&self.tc)
    
    def __repr__(self):
        return "Timecode('%s')" % self.__str__()


def _test():
    for args in ((4444,),(5,4,3,2),("99:54:32:10",)):
        print "arguments are: %s" % (args,)
        for tctype in ('30','25','24','60','30DF'):
            t = apply(Timecode,args,{'timecodetype':tctype})
            print tctype,t,int(t)
        print ""
    # assert (Timecode(1) + Timecode(2)) == Timecode(3) # not yet
    
